"use strict";
const color_1 = require("@oclif/color");
const command_1 = require("@oclif/command");
const cli_progress_1 = require("cli-progress");
const fs_1 = require("fs");
const recursiveReadDir = require("recursive-readdir");
const ts_simple_ast_1 = require("ts-simple-ast");
const angular_parser_1 = require("./angular-parser");
class NgxBindIoCli extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(NgxBindIoCli);
        if (!args.path) {
            console.log(color_1.color.red('Please set path with sources for scan, example: npm ngx-bind-io ./src'));
        }
        if (args.path) {
            const maxCountOfInputs = flags.maxInputs === undefined ? 3 : flags.maxInputs;
            const maxCountOfOutputs = flags.maxOutputs === undefined ? 3 : flags.maxOutputs;
            const info = flags.info;
            const verbose = flags.verbose;
            const fix = flags.fix;
            const path = args.path;
            const tsconfig = flags.tsconfig || '';
            const ignores = flags.ignores || [
                'node_modules',
                'dist',
                '*.spec.*',
                '.git'
            ];
            this.showInfo(path, tsconfig, ignores, fix, info, verbose, maxCountOfInputs, maxCountOfOutputs);
            const files = await this.scanFiles(path, ignores);
            const sources = this.loadSources(files);
            const components = this.parseComponentsFromFiles(path, tsconfig, files, fix);
            const useds = this.countUsages(components, files, sources);
            this.showResults(useds, maxCountOfInputs, maxCountOfOutputs, info, verbose);
        }
    }
    showResults(useds, maxCountOfInputs, maxCountOfOutputs, info, verbose) {
        let readyForBindIOCount = 0;
        let notReadyForBindIOCount = 0;
        useds.forEach(used => {
            if (used.usedCount !== undefined &&
                used.usedCount > 0) {
                const noInitializedInputs = used.inputs.filter(input => !input.initialized);
                const noInitializedOutputs = used.outputs.filter(output => !output.initialized);
                const noInitializedIO = noInitializedInputs.length > maxCountOfInputs ||
                    noInitializedOutputs.length > maxCountOfOutputs;
                const autoInitializedIO = used.autoInitializedInputs.filter(input => !input.initialized).length > 0;
                const groupName = (noInitializedIO ?
                    color_1.color.yellow('<' + used.selector + '>') :
                    color_1.color.green('<' + used.selector + '>')) +
                    ': ';
                if (noInitializedIO || autoInitializedIO || info) {
                    if (verbose) {
                        console.log(groupName +
                            `Class: ${color_1.color.blueBright(used.className)}`);
                        console.log(groupName +
                            `File: ${color_1.color.blueBright(used.file)}`);
                        if (used.usedCount > 0) {
                            console.log(groupName +
                                `Used in ${color_1.color.blueBright(String(used.usedCount))} places (all)`);
                        }
                        if (used.withBindIOCount && used.withBindIOCount > 0) {
                            console.log(groupName +
                                `Used with "bindIO" directive in ${color_1.color.greenBright(String(used.withBindIOCount))} places`);
                        }
                        if (used.withBindInputsCount && used.withBindInputsCount > 0) {
                            console.log(groupName +
                                `Used with "bindInputs" directive in ${color_1.color.greenBright(String(used.withBindInputsCount))} places`);
                        }
                        if (used.withBindOutputsCount && used.withBindOutputsCount > 0) {
                            console.log(groupName +
                                `Used with "bindOutputs" directive in ${color_1.color.greenBright(String(used.withBindOutputsCount))} places`);
                        }
                    }
                    if (info && used.inputs.length > 0) {
                        console.log(groupName +
                            `Inputs: ${used.inputs.map(input => color_1.color.blueBright(input.name)).join(', ')}`);
                    }
                    if (used.autoInitializedInputs.length > 0) {
                        console.log(groupName +
                            `Auto initialized inputs: ${used.autoInitializedInputs.map(input => color_1.color.yellowBright(input.name)).join(', ')}`);
                    }
                    if (info && used.outputs.length > 0) {
                        console.log(groupName +
                            `Outputs: ${used.outputs.map(output => color_1.color.blueBright(output.name)).join(', ')}`);
                    }
                    if (noInitializedInputs.length > maxCountOfInputs) {
                        console.log(groupName +
                            color_1.color.red(`Not initialized inputs: ${noInitializedInputs.map(input => color_1.color.redBright(input.name)).join(', ')}`));
                    }
                    if (noInitializedOutputs.length > maxCountOfOutputs) {
                        console.log(groupName +
                            color_1.color.red(`Not initialized outputs: ${noInitializedOutputs.map(output => output.name).join(', ')}`));
                    }
                    if (noInitializedIO) {
                        notReadyForBindIOCount++;
                    }
                    else {
                        readyForBindIOCount++;
                    }
                }
            }
        });
        if (readyForBindIOCount > 0) {
            console.log(color_1.color.green(`Ready for migrate to NgxBindIO: ${readyForBindIOCount} components`));
        }
        if (notReadyForBindIOCount > 0) {
            console.log(color_1.color.red(`Not ready for migrate to NgxBindIO: ${notReadyForBindIOCount} components`));
        }
        if (readyForBindIOCount === 0 && notReadyForBindIOCount === 0) {
            console.log(color_1.color.blue(`Not need migrate to NgxBindIO`));
        }
    }
    countUsages(components, files, sources) {
        const bar4 = new cli_progress_1.Bar({
            format: color_1.color.yellow('(4/4)') + ' Count usages [{bar}] {percentage}% | {value}/{total}'
        }, cli_progress_1.Presets.shades_classic);
        bar4.start(components.length, 1);
        const useds = components.map((component, index) => {
            files.map((file, index) => {
                const counts = angular_parser_1.countUsed(sources[index], component);
                component.usedCount = (component.usedCount || 0) + counts.allCount;
                component.withBindIOCount = (component.withBindIOCount || 0) + counts.withBindIOCount;
                component.withBindInputsCount = (component.withBindInputsCount || 0) + counts.withBindInputsCount;
                component.withBindOutputsCount = (component.withBindOutputsCount || 0) + counts.withBindOutputsCount;
            });
            bar4.update(index + 1);
            return component;
        });
        bar4.stop();
        return useds;
    }
    parseComponentsFromFiles(path, tsconfig, files, fix) {
        let components = [];
        const tsFiles = files.filter(file => file.indexOf('.ts') === file.length - 3);
        const bar3 = new cli_progress_1.Bar({
            format: color_1.color.yellow('(3/4)') + ' Parse components [{bar}] {percentage}% | ETA: {eta}s | {value}/{total}'
        }, cli_progress_1.Presets.shades_classic);
        bar3.start(tsFiles.length, 1);
        const project = tsconfig ? new ts_simple_ast_1.default({
            tsConfigFilePath: tsconfig
        }) : new ts_simple_ast_1.default();
        project.addExistingSourceFiles(tsFiles);
        project.getSourceFiles().forEach((sourceFile, index) => {
            const componentsInFile = sourceFile ? angular_parser_1.getAngularComponents(project, sourceFile, fix) : [];
            components = [
                ...components,
                ...componentsInFile
            ];
            bar3.update(index + 1);
        });
        bar3.stop();
        return components;
    }
    loadSources(files) {
        const bar2 = new cli_progress_1.Bar({
            format: color_1.color.yellow('(2/4)') + ' Load sources [{bar}] {percentage}% | {value}/{total}'
        }, cli_progress_1.Presets.shades_classic);
        bar2.start(files.length, 1);
        const sources = files.map((file, index) => {
            const content = fs_1.readFileSync(file).toString();
            bar2.update(index + 1);
            return content;
        });
        bar2.stop();
        return sources;
    }
    async scanFiles(path, ignores) {
        const bar1 = new cli_progress_1.Bar({
            format: color_1.color.yellow('(1/4)') + ' Scan files [{bar}] {percentage}% | {value}/{total}'
        }, cli_progress_1.Presets.shades_classic);
        bar1.start(100, 10);
        const files = await recursiveReadDir(path, ignores);
        bar1.update(100);
        bar1.stop();
        return files;
    }
    showInfo(path, tsconfig, ignores, fix, info, verbose, maxCountOfInputs, maxCountOfOutputs) {
        console.log('Path: ' + color_1.color.blue(path));
        console.log('Ignores: ' + color_1.color.blue(ignores.join(', ')));
        if (tsconfig) {
            console.log('Typescript config: ' + color_1.color.blue(tsconfig));
        }
        if (fix) {
            console.log('Auto initialize: ' + color_1.color.blue('true'));
        }
        if (info) {
            console.log('Info: ' + color_1.color.blue('true'));
        }
        if (verbose) {
            console.log('Verbose: ' + color_1.color.blue('true'));
        }
        if (maxCountOfInputs !== undefined && maxCountOfInputs !== 3) {
            console.log('Max inputs count: ' + color_1.color.blue(String(maxCountOfInputs)));
        }
        if (maxCountOfOutputs !== undefined && maxCountOfOutputs !== 3) {
            console.log('Max outputs count: ' + color_1.color.blue(String(maxCountOfOutputs)));
        }
    }
}
NgxBindIoCli.description = 'Tools for check Angular7+ components for use ngx-bind-io directives';
NgxBindIoCli.flags = {
    version: command_1.flags.version({ char: 'v' }),
    help: command_1.flags.help({ char: 'h' }),
    info: command_1.flags.boolean({ char: 'i', description: 'Show inputs and outputs used in components', default: false }),
    verbose: command_1.flags.boolean({ char: 'V', description: 'Show all detail informations for inputs and outputs used in components', default: false }),
    fix: command_1.flags.boolean({ char: 'f', description: 'Auto initialized all not initialized inputs', default: false }),
    ignores: command_1.flags.string({
        multiple: true, description: 'Ignored files and paths'
    }),
    maxInputs: command_1.flags.integer({ description: 'Max count of inputs for detect need use NgxBindIO directives', default: 3 }),
    maxOutputs: command_1.flags.integer({ description: 'Max count of outputs for detect need use NgxBindIO directives', default: 3 }),
    tsconfig: command_1.flags.string({ char: 'c', description: 'Please set if you use tspaths for correct scan base components' }),
};
NgxBindIoCli.args = [{ name: 'path' }];
module.exports = NgxBindIoCli;
